--[[
 MiniPet Addon
 Version 3.1
 $Revision: 76448 $

 Currently maintained by Breanni of Scarlet Crusade
 All credit for authoring this addon is listed below

 Visit WarcraftPets.com for all your minipet needs!
]]--

--[[
    MiniPet by Carra of Stormrage
    
    There are a number of events where the player looses his pet and we can notify him:
    -Entering the game                   >OK
    -Entering an instance                >OK
    -Dieing and getting back alive       >Usually OK :p
    -Using a Taxi Service                >OK
    -Teleporting/Summoning/Hearthstoning >OK but no summoning
    
    Based originally on MiniPetLeash (completely rewritten except for the events which are copied).
    
    Updated and converted to Ace2 by LordFarlander
]]--

local L = AceLibrary( "AceLocale-2.2" ):new( "MiniPet" );
local LBZ = AceLibrary( "LibBabble-Zone-3.0" ):GetLookupTable();

_G["BINDING_NAME_CLICK MiniPet_Button:LeftButton"] = L["Summon MiniPet"];
BINDING_HEADER_MINIPETHEADER = L["MiniPet"];

local options = {
    type = "group",
    args = {
        [L["toggleAutoHideButton"]] = {
            type = "toggle",
            name = L["Auto Hide Button"],
            desc = L["Hide the button when you clicked it?"],
            get = function()
                      return MiniPet.db.char.AutoHideButton;
                  end,
            set = function( val )
                      MiniPet.db.char.AutoHideButton = not MiniPet.db.char.AutoHideButton;
                  end,
        },
--        toggleDebug = {
--            type = "toggle",
--            name = "Debug",
--            desc = "Debug mode toggle",
--            get = function()
--                      return MiniPet.DEBUG;
--                  end,
--            set = function( val )
--                      MiniPet.DEBUG = not MiniPet.DEBUG;
--                  end,
--            },
        [L["toggleButton"]] = {
            type = "toggle",
            name = L["Show Button"],
            desc = L["Show/Hide the MiniPet-Button."],
            get = function()
                      return MiniPet.button:IsVisible()
                  end,
            set = function( val )
                      if( MiniPet.button:IsVisible() ) then
                          MiniPet.button:Hide();
                      else
                          MiniPet.button:Show();
                      end
                  end,
            },
        [L["toggleTextWarning"]] = {
            type = "toggle",
            name = L["Text Warning"],
            desc = L["Show text warnings to warn you get out a minipet?"],
            get = function()
                      return MiniPet.db.char.GiveTextWarning;
                  end,
            set = function( val )
                      MiniPet.db.char.GiveTextWarning = not MiniPet.db.char.GiveTextWarning;
                  end,
            },
        [L["toggleButtonWarning"]] = {
            type = "toggle",
            name = L["Button Warning"],
            desc = L["Flash the button to warn you to get out a minipet?"],
            get = function()
                      return MiniPet.db.char.GiveButtonWarning;
                  end,
            set = function( val )
                      MiniPet.db.char.GiveButtonWarning = not MiniPet.db.char.GiveButtonWarning;
                  end,
            },
        [L["toggleMountWarning"]] = {
            type = "toggle",
            name = L["Mount Warning"],
            desc = L["Show text warnings to warn you get out a minipet when getting of a flying mount?"],
            get = function()
                      return MiniPet.db.char.GiveMountWarning;
                  end,
            set = function( val )
                      MiniPet.db.char.GiveMountWarning = not MiniPet.db.char.GiveMountWarning;
                  end,
            },
        [L["toggleBG"]] = {
            type = "toggle",
            name = L["Battle Ground Check"],
            desc = L["Show warnings in a battleground?"],
            get = function()
                      return MiniPet.db.char.BattleGroundCheck;
                  end,
            set = function( val )
                      MiniPet.db.char.BattleGroundCheck = not MiniPet.db.char.BattleGroundCheck;
                  end,
            },
        [L["lockButton"]] = {
            type = "toggle",
            name = L["Lock Button Position"],
            desc = L["Lock the position of the button?"],
            get = function()
                      return MiniPet.db.char.LockButton;
                  end,
            set = function( val )
                      MiniPet.db.char.LockButton = not MiniPet.db.char.LockButton;
                  end,
             },
        [L["suggestOnlySoulbound"]] = {
            type = "toggle",
            name = L["Suggest Only Soulbound MiniPets"],
            desc = L["Toggles if only minipets that are soulbound to you are to be suggested for use."],
            get = function()
                      return MiniPet.db.char.SuggestOnlySoulbound;
                  end,
            set = function( val )
                      MiniPet.db.char.SuggestOnlySoulbound = not MiniPet.db.char.SuggestOnlySoulbound;
                      MiniPet:GetPlayersPets();
                      MiniPet:ChooseRandomPet();
                  end,
            },
        [L["reagentCheck"]] = {
            type = "toggle",
            name = L["Check for reagented MiniPets that are missing their reagent"],
            desc = L["Toggles if minipets that require a reagent, and you have none of that reagent, are to be shown in the list of pets and randomly picked from."],
            get = function()
                      return MiniPet.db.char.ReagentCheck;
                  end,
            set = function( val )
                      MiniPet.db.char.ReagentCheck = not MiniPet.db.char.ReagentCheck;
                      MiniPet:GetPlayersPets();
                      MiniPet:ChooseRandomPet();
                  end,
            },
        [L["equipmentCheck"]] = {
            type = "toggle",
            name = L["Check to make sure that equipment MiniPets are equipped"],
            desc = L["Toggles if minipets that need to be equipped are before adding them to your list of pets."],
            get = function()
                      return MiniPet.db.char.EquipmentCheck;
                  end,
            set = function( val )
                      MiniPet.db.char.EquipmentCheck = not MiniPet.db.char.EquipmentCheck;
                      MiniPet:GetPlayersPets();
                      MiniPet:ChooseRandomPet();
                  end,
            },
        [L["forceQuestPet"]] = {
            type = "toggle",
            name = L["Force to pick a quest MiniPet if availiable"],
            desc = L["Toggles if minipets that are part of a quest are picked instead of a random pet."],
            get = function()
                      return MiniPet.db.char.ForceQuestPet;
                  end,
            set = function( val )
                      MiniPet.db.char.ForceQuestPet = not MiniPet.db.char.ForceQuestPet;
                      MiniPet:GetPlayersPets();
                      MiniPet:ChooseRandomPet();
                  end,
            },
        [L["list"]] = {
            type = "execute",
            name = L["List all MiniPets"],
            desc = L["Show a list with all your minipets."],
            func = "ListPets",
            },
        [L["relist"]] = {
            type = "execute",
            name = L["Refresh MiniPet list"],
            desc = L["Force a refresh of the list of all your minipets."],
            func = "GetPlayersPets",
            },
        },
    };

MiniPet = AceLibrary( "AceAddon-2.0" ):new( "AceConsole-2.0", "AceEvent-2.0", "AceDB-2.0" );
MiniPet.version = tonumber( string.match( "$Rev: 76448 $", "(%d+)" ) );
MiniPet:RegisterChatCommand( "/MiniPet", "/mp", options );
MiniPet:RegisterDB( "MiniPetDB", "MiniPetDBPC" );
MiniPet:RegisterDefaults( "char", {
        AutoHideButton = true,
        BattleGroundCheck = true,
        GiveTextWarning = true,
        GiveButtonWarning = true,
        GiveMountWarning = true,
        LockButton = false,
        SuggestOnlySoulbound = true,
        ReagentCheck = true,
        EquipmentCheck = true,
        ForceQuestPet = true,
        ButtonGroup = {
            SkinID = "Blizzard",
            Gloss = nil,
            Backdrop = nil,
            Colors = {},
            },
    } );

local PetAndMountDatabase = LibStub( "PetAndMountDatabase-1.0", true );
if( not PetAndMountDatabase ) then
    error( "MiniPet requires PetAndMountDatabase-1.0" );
    return;
end--if

local LibHelper = LibStub( "LibHelper-1.0", true );
if( not LibHelper ) then
    error( "MiniPet requires LibHelper-1.0" );
    return;
end--if

local ItemList = LibStub( "LibHelper-ItemList-1.0", true );
if( not ItemList ) then
    error( "MiniPet requires LibHelper-ItemList-1.0" );
    return;
end--if

local LibButtonFacade = LibStub( "LibButtonFacade", true );
        
--Global Values
MiniPet.DEBUG = false;            --true = Debugging Mode
MiniPet.dprint = LibHelper.dprint;
MiniPet.button = nil;

MiniPet.UpdateInterval      = 0.2;   --Time between two flashes of the button in seconds
MiniPet.TotalFlashTime      = 10;    --Total time to flash the button in seconds

--DO NOT CHANGE THESE VALUES
MiniPet.PlayersPets         = {};    --List of the players pet
MiniPet.CurrentPet          = nil;   --Pet on the Button
MiniPet.BoardedTaxi         = false;
MiniPet.StartedHearthstone  = false;
MiniPet.HearthstoneComplete = false;
MiniPet.PlayerDied          = false;
MiniPet.PlayerEnteringWorld = true;
MiniPet.ReadyToInitialize   = false;
MiniPet.PetListOkay         = false;
MiniPet.LastPet             = nil;
MiniPet.QuestPet            = nil;

MiniPet.ShowButtonImg       = false; --Goes on and off if button flashes
MiniPet.FlashTime           = 0;     --Time since button flashes
MiniPet.FlashButton         = false; --On when the button flashes

MiniPet.Dirty               = false;

local MINIPET_BATTLEGROUNDS = table.concat( {
    	LBZ["Warsong Gulch"],
    	LBZ["Arathi Basin"],
    	LBZ["Alterac Valley"],
    	LBZ["Eye of the Storm"],
	}, "|" );

function MiniPet:OnEnable()
    --Register the Events
    self:RegisterEvent( "BAG_UPDATE",
        function( bagID )
            MiniPet.Dirty = true;
        end );
    self:RegisterEvent( "PLAYER_ENTERING_WORLD",
        function()
            MiniPet.PlayerDied = UnitIsDeadOrGhost( "player" );
            MiniPet.BoardedTaxi = UnitOnTaxi( "player" );
            MiniPet.PlayerEnteringWorld = false;
            MiniPet:WarnPlayer();
        end );
    self:RegisterEvent( "TAXIMAP_CLOSED",
        function()
            MiniPet.BoardedTaxi = true;
        end );
    self:RegisterEvent( "PLAYER_CONTROL_GAINED",
        function()
            if( MiniPet.BoardedTaxi ) then
                MiniPet.BoardedTaxi = false;
                MiniPet:WarnPlayer();
            end--if
        end );
    self:RegisterEvent( "PLAYER_DEAD",
        function()
            MiniPet:dprint( "MiniPet: PlayerDied" );
            MiniPet.PlayerDied = true;
        end );
    self:RegisterEvent( "PLAYER_ALIVE",
        function()
            MiniPet.BoardedTaxi = UnitOnTaxi( "player" );
            if( MiniPet.HearthstoneComplete ) then
                MiniPet.HearthstoneComplete = false;
                MiniPet:dprint( "MiniPet: Activating Pet After Hearthstone!" );
                MiniPet:WarnPlayer();
            end--if
            if( MiniPet.PlayerDied ) then
                MiniPet.PlayerDied = false;
                MiniPet:dprint( "MiniPet: no longer dead!" );
                MiniPet:WarnPlayer();
            end--if
        end );
    self:RegisterEvent( "PLAYER_UNGHOST",
        function()
            if( MiniPet.PlayerDied ) then
                MiniPet.PlayerDied = false;
                MiniPet:dprint( "MiniPet: no longer ghost!" );
                MiniPet:WarnPlayer();
            end--if
        end );
    self:RegisterEvent( "UNIT_SPELLCAST_INTERRUPTED",
        function( unit, _, _ )
            if( unit == "player" ) then
                if( MiniPet.StartedHearthstone ) then
                    MiniPet:dprint( "MiniPet: Interrupted Teleporting!" );
                    MiniPet.StartedHearthstone = false;
                end--if
            end--if
        end );
    self:RegisterEvent( "UNIT_SPELLCAST_SUCCEEDED",
        function( unit, spellName, _ )
            if( unit == "player" ) then
                if( MiniPet.StartedHearthstone ) then
                    if( MiniPet.IsATeleportSpell( spellName ) ) then
                        MiniPet:dprint( "MiniPet: Finished Teleporting!" );
                        MiniPet.StartedHearthstone = false;
                        MiniPet.HearthstoneComplete = true;
                        MiniPet:WarnPlayer();
                    end--if
                end--if
            end--if
        end );
    self:RegisterEvent( "UNIT_SPELLCAST_SENT",
        function( unit, spellName, _, _ )
            if( unit == "player" ) then
                MiniPet:dprint( "Spellcast: " .. spellName );
                MiniPet.StartedHearthstone = false;
                if( MiniPet.IsATeleportSpell( spellName ) ) then
                    MiniPet:dprint( "MiniPet: Started Teleporting!" );
                    MiniPet.StartedHearthstone = true;
                end--if
            end--if
        end );
    self:RegisterEvent( "SpecialEvents_Dismounted",
        function( mountName, speed, flying )
            if( flying and MiniPet.db.char.GiveMountWarning ) then
                MiniPet:WarnPlayer();
            end--if
        end );
    self:RegisterEvent( "PLAYER_REGEN_ENABLED",
        function()
            MiniPet:SetButton( MiniPet.CurrentPet );
        end );

    self.button = MiniPet_Button;
    self.button.tooltip = MiniPet_Tooltip;

    self.button.tooltip.SetFromListEntry = ItemList.SetTooltipFromListEntry;
    self.button.SetFromListEntry = ItemList.SetButtonFromListEntry;
    self.button.SetImage = LibHelper.SetButtonImage;
    self.button.cooldown = _G[self.button:GetName() .. "Cooldown"];

    if( LibButtonFacade ) then
        self.button.icon = _G[self.button:GetName() .. "Icon"];
        LibButtonFacade:Group( "MiniPet" ):AddButton( self.button );
        LibButtonFacade:Group( "MiniPet" ):Skin( self.db.char.ButtonGroup.SkinID, self.db.char.ButtonGroup.Gloss, self.db.char.ButtonGroup.Backdrop, self.db.char.ButtonGroup.Colors );
        LibButtonFacade:RegisterSkinCallback( "MiniPet", self.SkinChanged, self );
    end--if

    --Upgrade check
    if( self.db.char.LastVersion == nil ) then
        --Upgrading from 2.0 or earlier, import the old settings
        self.db.char.BattleGroundCheck = MiniPet_BattleGroundCheck;
        self.db.char.GiveTextWarning = MiniPet_GiveTextWarning;
        self.db.char.GiveButtonWarning = MiniPet_GiveButtonWarning;
        self.db.char.AutoHideButton = MiniPet_HideButton;
    end--if
    if( self.db.char.LastVersion ~= self.version ) then
        self.db.char.LastVersion = self.version;
    end--if

    self.Dirty = true;
    --Create the Pet Inventory
    self:GetPlayersPets();
    --Set a Pet
    self:ChooseRandomPet();
end--MiniPet:OnEnable();

--For ButtonFacade support
function MiniPet:SkinChanged( SkinID, Gloss, Backdrop, barKey, buttonKey, Colors )
    self.db.char.ButtonGroup.SkinID = SkinID;
    self.db.char.ButtonGroup.Gloss = Gloss;
    self.db.char.ButtonGroup.Backdrop = Backdrop;
    self.db.char.ButtonGroup.Colors = Colors;
end--MiniPet:SkinChanged( SkinID, Gloss, Backdrop, barKey, buttonKey, Colors )

function MiniPet:OnUpdate( elapsed )
    if( ( UnitName( "player" ) ~= UNKNOWNOBJECT ) and self.ReadyToInitialize ) then
        self:dprint( "MiniPet: PlayerEnteringWorld and ALIVE" );
        self.PlayerEnteringWorld = false;
        self.ReadyToInitialize = false;
        self:WarnPlayer();
    end
    
    if( self.Dirty ) then
        self.Dirty = false;
        if( self:HasMiniPetListChanged() ) then
            self:ChooseRandomPet();
        end--if
        self:SetButton( self.CurrentPet );
    end--if

    --Code to let the button flash
    --elapsed = amount of seconds that passed since last update.
    --every updateinterval, change the button image
    --until you reach totalflashtime
    if( self.FlashButton ) then
        if( self.button.TimeSinceLastUpdate == nil ) then
            self.button.TimeSinceLastUpdate = 0;
            self.FlashTime = 0;
        end--if
        self.button.TimeSinceLastUpdate = self.button.TimeSinceLastUpdate + elapsed;
        self.FlashTime = self.FlashTime + elapsed;
        if( self.FlashTime < self.TotalFlashTime ) then
            if( self.button.TimeSinceLastUpdate > self.UpdateInterval ) then
                if( self.ShowButtonImg and ( self.CurrentPet ~= nil ) ) then
                    self.button:SetFromListEntry( self.CurrentPet );
                    self.ShowButtonImg = false;
                else
                    self.button:SetImage( "Interface\\AddOns\\MiniPet\\Img\\MiniPet" );
                    self.ShowButtonImg = true;
                end--if
                self.button.TimeSinceLastUpdate = 0;
            end--if
        else
            self.FlashButton = false;
            self:SetButton( self.CurrentPet );
            self.FlashTime = 0;
        end--if
    end--if
end--MiniPet:OnUpdate( elapsed )

----------------------------
--+Pet/Inventory Functions--
--Pick a random pet and set the button.
function MiniPet:ChooseRandomPet()
    if( self.db.char.ForceQuestPet and self.QuestPet ) then
        self.CurrentPet = self.QuestPet;
    else
        self.CurrentPet = ItemList.GetRandomItemFromList( self.PlayersPets, self.CurrentPet );
    end--if
    self:SetButton( self.CurrentPet );
end--MiniPet:ChooseRandomPet()

--Gets all your pets from your inventory.
function MiniPet:GetPlayersPets()
    self.PlayersPets = {};
    self.QuestPet = nil;

    local playerPetList = self.PlayersPets;
    local itemLink;
    local reagented;
    local equipmentSlot;
    local opSuggestOnlySoulbound = self.db.char.SuggestOnlySoulbound;
    local opReagentCheck = self.db.char.ReagentCheck;
    local opEquipmentCheck = self.db.char.EquipmentCheck;

    for bag = 0, NUM_BAG_SLOTS do
        for slot = 1, GetContainerNumSlots( bag ) do
            itemLink = GetContainerItemLink( bag, slot );
            if( PetAndMountDatabase:ItemInSet( itemLink, "MiniPets" ) ) then
                if( opReagentCheck ) then
                    reagented = PetAndMountDatabase:ItemInSet( itemLink, "MiniPets.Reagented" );
                else
                    reagented = nil;
                end--if
                if( ( ( not opSuggestOnlySoulbound ) or ( LibHelper.ItemIsSoulbound( bag, slot ) ) ) and
                    ( ( not opReagentCheck ) or ( not reagented ) or ( GetItemCount( reagented ) > 0 ) ) and
                    ( ( not opEquipmentCheck ) or ( not PetAndMountDatabase:ItemInSet( itemLink, "MiniPets.Equipment" ) ) ) ) then
                    ItemList.AddBagItemToList( playerPetList, bag, slot );
                    if( self.db.char.ForceQuestPet ) then
                        local questID = PetAndMountDatabase:ItemInSet( itemLink, "MiniPets.Quest" );

                        if( ( questID and self:HasQuestAndNotComplete( questID ) ) or
                            ( ( self.QuestPet == nil ) and PetAndMountDatabase:ItemInSet( itemLink, "MiniPets.Children" ) ) ) then
                            self.QuestPet = ItemList.BuildBagItem( bag, slot );
                        end--if
                    end--if
                end--if
            end--if
        end--for
    end--for
    
    local iterator, handle = PetAndMountDatabase:IterateSet( "MiniPets.Equipment" );
    local pet, equipmentSlot = iterator( handle );

    while pet do
        if( opEquipmentCheck and ( LibHelper.GetItemIDFromLink( GetInventoryItemLink( "player", GetInventorySlotInfo( equipmentSlot ) ) ) == pet ) ) then
            ItemList.AddInventoryItemToList( playerPetList, equipmentSlot );
        end--if
        pet, equipmentSlot = iterator( handle );
    end--for
end--MiniPet:GetPlayerPets()

--Prints out a list with all your pets.
function MiniPet:ListPets()
    self:Print( L["MiniPet, List of your pets:"] );
    for _, pet in pairs( self.PlayersPets ) do
        self:Print( "-|cFF00FFFF" .. pet:GetName() );
    end--for
end--MiniPet:listPets()
---Pet/Invenroty Functions--
----------------------------

-------------------
--+Help Functions--
function MiniPet.IsATeleportSpell( spellName )
    local iterator, handle = PetAndMountDatabase:IterateSet( "Spells.Teleport" );
    local spell, _ = iterator( handle );

    while spell do
        if( spell < 0 ) then
            spell = -1 * spell;
        end--if
        if( spellName == GetSpellInfo( spell ) ) then
            return true;
        end--if
        spell, _ = iterator( handle );
    end--for
    return false;
end--MiniPet.IsATeleportSpell( spellName )

function MiniPet:HasMiniPetListChanged()
    local prePetCount = #self.PlayersPets;
    local prePetList =  {};

    LibHelper.CopyTable( prePetList, self.PlayersPets );

    self:GetPlayersPets();

    -- differences in sizes always means the list has changed
    if( ( prePetCount ~= #self.PlayersPets ) or ( not self.PetListOkay ) ) then
        self.PetListOkay = true;
        self:dprint( "Number of minipets changed" );
        return true;
    end--if
    self.PetListOkay = true;
    return not LibHelper.CompareTables( prePetList, self.PlayersPets );
end--MiniPet:HasMiniPetListChanged()

--Check if a player is in a battleground
function MiniPet.IsInBattleground()
    return MINIPET_BATTLEGROUNDS:find( GetZoneText() ) ~= nil;
end--MiniPet.IsInBattleground()

function MiniPet.HasQuestAndNotComplete( questID )
	local numEntries, _ = GetNumQuestLogEntries();

	if( numEntries > 0 ) then
		for id = 1, numEntries do
            local _, _, _, _, _, _, complete, _ = GetQuestLogTitle( id );
            
            if( ( not complete ) and ( LibHelper.GetItemIDFromLink( GetQuestLink( id ) ) == questID ) ) then
                return true;
            end--if
        end--for
    end--if
    return false;
end--Minipet.GetQuestID( questName )
---Help Functions--
-------------------

----------------------
--+Message Functions--
--Give the player a reminder.
function MiniPet:WarnPlayer()
    local charDB = self.db.char;

    if( ( not ( self.IsInBattleground() and charDB.BattleGroundCheck ) ) and ( not self.PlayerDied ) and ( not self.BoardedTaxi ) ) then--BG + option check
        self.FlashButton = charDB.GiveButtonWarning;
        if( charDB.GiveTextWarning ) then
            if( #self.PlayersPets > 0 ) then    --No use if player isn't cool and gots no pets!
                self.wprint( L["ALERT: Time to summon your minipet!"] );
            end--if
        end--if
        if( charDB.AutoHideButton and ( not self.button:IsVisible() ) ) then
            self.button:Show();
        end--if
    end--if
end--MiniPet:WarnPlayer()

--Shows a warning in the middle of the screen.
function MiniPet.wprint( msg )
    MinipetTextFrameText:SetTextColor( 1.0, 0.1, 0.1 );
    MinipetTextFrameText:SetText( msg );
    MinipetTextFrame.startTime = GetTime();
    MinipetTextFrame:Show();
end--MiniPet:wprint( msg )
---Message Functions
----------------------

-----------------
--+UI Functions--
function MiniPet:SetGameTooltips()
    local pet = self.CurrentPet;
    local mpButton = self.button;    
    local mpTooltip = mpButton.tooltip;

    mpTooltip:SetOwner( mpButton, "RIGHT" );
    mpTooltip:SetBackdropColor( 0.0, 0.0, 0.0, 1.0 );
    if( pet == nil ) then
        mpTooltip:SetText( L["Get a minipet for some company!"] );
    else
        mpTooltip:SetFromListEntry( pet );
    end--if
end--MiniPet:SetGameTooltips()

function MiniPet:SetButton( pet )
    local mpButton = self.button;

    if( not UnitAffectingCombat( "player" ) ) then
        if( pet == nil ) then
            mpButton.cooldown:Hide();
            mpButton:SetAttribute( "type1", "item" );
            mpButton:SetAttribute( "*bag1", nil );
            mpButton:SetAttribute( "*slot1", nil );
            mpButton:SetAttribute( "*item1", nil );

            mpButton:SetImage( "Interface\\AddOns\\MiniPet\\Img\\MiniPet" );

            self:dprint( "No pets" );
        else
            mpButton:SetFromListEntry( pet, not LibHelper.CompareTables( self.LastPet, pet ) );
        end--if
        self.LastPet = pet;
    end--if
end--MiniPet:SetButton( pet )
---UI Functions--
-----------------
